@@ -11,16 +11,18 @@ import difflib
 import re
 import struct
 
+
 def splitnewlines(text):
     '''like str.splitlines, but only split on newlines.'''
-    lines = [l + '\n' for l in text.split('\n')]
+    lines = [l + b'\n' for l in text.split(b'\n')]
     if lines:
-        if lines[-1] == '\n':
+        if lines[-1] == b'\n':
             lines.pop()
         else:
             lines[-1] = lines[-1][:-1]
     return lines
 
+
 def _normalizeblocks(a, b, blocks):
     prev = None
     r = []
@@ -38,25 +40,31 @@ def _normalizeblocks(a, b, blocks):
         a2end = a2 + l2
         b2end = b2 + l2
         if a1end == a2:
-            while (a1end + shift < a2end and
-                   a[a1end + shift] == b[b1end + shift]):
+            while (
+                a1end + shift < a2end and a[a1end + shift] == b[b1end + shift]
+            ):
                 shift += 1
         elif b1end == b2:
-            while (b1end + shift < b2end and
-                   a[a1end + shift] == b[b1end + shift]):
+            while (
+                b1end + shift < b2end and a[a1end + shift] == b[b1end + shift]
+            ):
                 shift += 1
         r.append((a1, b1, l1 + shift))
         prev = a2 + shift, b2 + shift, l2 - shift
-    r.append(prev)
+
+    if prev is not None:
+        r.append(prev)
+
     return r
 
+
 def bdiff(a, b):
     a = bytes(a).splitlines(True)
     b = bytes(b).splitlines(True)
 
     if not a:
-        s = "".join(b)
-        return s and (struct.pack(">lll", 0, 0, len(s)) + s)
+        s = b"".join(b)
+        return s and (struct.pack(b">lll", 0, 0, len(s)) + s)
 
     bin = []
     p = [0]
@@ -68,13 +76,14 @@ def bdiff(a, b):
     la = 0
     lb = 0
     for am, bm, size in d:
-        s = "".join(b[lb:bm])
+        s = b"".join(b[lb:bm])
         if am > la or s:
-            bin.append(struct.pack(">lll", p[la], p[am], len(s)) + s)
+            bin.append(struct.pack(b">lll", p[la], p[am], len(s)) + s)
         la = am + size
         lb = bm + size
 
-    return "".join(bin)
+    return b"".join(bin)
+
 
 def blocks(a, b):
     an = splitnewlines(a)
@@ -83,10 +92,11 @@ def blocks(a, b):
     d = _normalizeblocks(an, bn, d)
     return [(i, i + n, j, j + n) for (i, j, n) in d]
 
+
 def fixws(text, allws):
     if allws:
-        text = re.sub('[ \t\r]+', '', text)
+        text = re.sub(b'[ \t\r]+', b'', text)
     else:
-        text = re.sub('[ \t\r]+', ' ', text)
-        text = text.replace(' \n', '\n')
+        text = re.sub(b'[ \t\r]+', b' ', text)
+        text = text.replace(b' \n', b'\n')
     return text
