@@ -8,6 +8,7 @@
 from __future__ import absolute_import
 
 import errno
+import getpass
 import msvcrt
 import os
 import re
@@ -16,6 +17,7 @@ import string
 import sys
 
 from .i18n import _
+from .pycompat import getattr
 from . import (
     encoding,
     error,
@@ -25,12 +27,14 @@ from . import (
 )
 
 try:
-    import _winreg as winreg
+    import _winreg as winreg  # pytype: disable=import-error
+
     winreg.CloseKey
 except ImportError:
-    import winreg
+    # py2 only
+    import winreg  # pytype: disable=import-error
 
-osutil = policy.importmod(r'osutil')
+osutil = policy.importmod('osutil')
 
 getfsmountpoint = win32.getvolumename
 getfstype = win32.getfstype
@@ -49,6 +53,7 @@ unlink = win32.unlink
 
 umask = 0o022
 
+
 class mixedfilemodewrapper(object):
     """Wraps a file handle when it is opened in read/write mode.
 
@@ -61,13 +66,14 @@ class mixedfilemodewrapper(object):
     mode and automatically adds checks or inserts appropriate file positioning
     calls when necessary.
     """
+
     OPNONE = 0
     OPREAD = 1
     OPWRITE = 2
 
     def __init__(self, fp):
-        object.__setattr__(self, r'_fp', fp)
-        object.__setattr__(self, r'_lastop', 0)
+        object.__setattr__(self, '_fp', fp)
+        object.__setattr__(self, '_lastop', 0)
 
     def __enter__(self):
         self._fp.__enter__()
@@ -86,48 +92,50 @@ class mixedfilemodewrapper(object):
         self._fp.seek(0, os.SEEK_CUR)
 
     def seek(self, *args, **kwargs):
-        object.__setattr__(self, r'_lastop', self.OPNONE)
+        object.__setattr__(self, '_lastop', self.OPNONE)
         return self._fp.seek(*args, **kwargs)
 
     def write(self, d):
         if self._lastop == self.OPREAD:
             self._noopseek()
 
-        object.__setattr__(self, r'_lastop', self.OPWRITE)
+        object.__setattr__(self, '_lastop', self.OPWRITE)
         return self._fp.write(d)
 
     def writelines(self, *args, **kwargs):
         if self._lastop == self.OPREAD:
             self._noopeseek()
 
-        object.__setattr__(self, r'_lastop', self.OPWRITE)
+        object.__setattr__(self, '_lastop', self.OPWRITE)
         return self._fp.writelines(*args, **kwargs)
 
     def read(self, *args, **kwargs):
         if self._lastop == self.OPWRITE:
             self._noopseek()
 
-        object.__setattr__(self, r'_lastop', self.OPREAD)
+        object.__setattr__(self, '_lastop', self.OPREAD)
         return self._fp.read(*args, **kwargs)
 
     def readline(self, *args, **kwargs):
         if self._lastop == self.OPWRITE:
             self._noopseek()
 
-        object.__setattr__(self, r'_lastop', self.OPREAD)
+        object.__setattr__(self, '_lastop', self.OPREAD)
         return self._fp.readline(*args, **kwargs)
 
     def readlines(self, *args, **kwargs):
         if self._lastop == self.OPWRITE:
             self._noopseek()
 
-        object.__setattr__(self, r'_lastop', self.OPREAD)
+        object.__setattr__(self, '_lastop', self.OPREAD)
         return self._fp.readlines(*args, **kwargs)
 
+
 class fdproxy(object):
     """Wraps osutil.posixfile() to override the name attribute to reflect the
     underlying file name.
     """
+
     def __init__(self, name, fp):
         self.name = name
         self._fp = fp
@@ -147,10 +155,11 @@ class fdproxy(object):
     def __getattr__(self, name):
         return getattr(self._fp, name)
 
-def posixfile(name, mode='r', buffering=-1):
+
+def posixfile(name, mode=b'r', buffering=-1):
     '''Open a file with even more POSIX-like semantics'''
     try:
-        fp = osutil.posixfile(name, mode, buffering) # may raise WindowsError
+        fp = osutil.posixfile(name, mode, buffering)  # may raise WindowsError
 
         # PyFile_FromFd() ignores the name, and seems to report fp.name as the
         # underlying file descriptor.
@@ -159,26 +168,44 @@ def posixfile(name, mode='r', buffering=-1):
 
         # The position when opening in append mode is implementation defined, so
         # make it consistent with other platforms, which position at EOF.
-        if 'a' in mode:
+        if b'a' in mode:
             fp.seek(0, os.SEEK_END)
 
-        if '+' in mode:
+        if b'+' in mode:
             return mixedfilemodewrapper(fp)
 
         return fp
     except WindowsError as err:
         # convert to a friendlier exception
-        raise IOError(err.errno, r'%s: %s' % (
-            encoding.strfromlocal(name), err.strerror))
+        raise IOError(
+            err.errno, '%s: %s' % (encoding.strfromlocal(name), err.strerror)
+        )
+
 
 # may be wrapped by win32mbcs extension
 listdir = osutil.listdir
 
+
+# copied from .utils.procutil, remove after Python 2 support was dropped
+def _isatty(fp):
+    try:
+        return fp.isatty()
+    except AttributeError:
+        return False
+
+
 class winstdout(object):
-    '''stdout on windows misbehaves if sent through a pipe'''
+    """Some files on Windows misbehave.
+
+    When writing to a broken pipe, EINVAL instead of EPIPE may be raised.
+
+    When writing too many bytes to a console at the same, a "Not enough space"
+    error may happen. Python 3 already works around that.
+    """
 
     def __init__(self, fp):
         self.fp = fp
+        self.throttle = not pycompat.ispy3 and _isatty(fp)
 
     def __getattr__(self, key):
         return getattr(self.fp, key)
@@ -191,12 +218,13 @@ class winstdout(object):
 
     def write(self, s):
         try:
+            if not self.throttle:
+                return self.fp.write(s)
             # This is workaround for "Not enough space" error on
             # writing large size of data to console.
             limit = 16000
             l = len(s)
             start = 0
-            self.softspace = 0
             while start < l:
                 end = start + limit
                 self.fp.write(s[start:end])
@@ -205,7 +233,7 @@ class winstdout(object):
             if inst.errno != 0 and not win32.lasterrorwaspipeerror(inst):
                 raise
             self.close()
-            raise IOError(errno.EPIPE, r'Broken pipe')
+            raise IOError(errno.EPIPE, 'Broken pipe')
 
     def flush(self):
         try:
@@ -213,76 +241,85 @@ class winstdout(object):
         except IOError as inst:
             if not win32.lasterrorwaspipeerror(inst):
                 raise
-            raise IOError(errno.EPIPE, r'Broken pipe')
+            raise IOError(errno.EPIPE, 'Broken pipe')
 
-def _is_win_9x():
-    '''return true if run on windows 95, 98 or me.'''
-    try:
-        return sys.getwindowsversion()[3] == 1
-    except AttributeError:
-        return 'command' in encoding.environ.get('comspec', '')
 
 def openhardlinks():
-    return not _is_win_9x()
+    return True
+
 
 def parsepatchoutput(output_line):
     """parses the output produced by patch and returns the filename"""
     pf = output_line[14:]
-    if pf[0] == '`':
-        pf = pf[1:-1] # Remove the quotes
+    if pf[0] == b'`':
+        pf = pf[1:-1]  # Remove the quotes
     return pf
 
+
 def sshargs(sshcmd, host, user, port):
     '''Build argument list for ssh or Plink'''
-    pflag = 'plink' in sshcmd.lower() and '-P' or '-p'
-    args = user and ("%s@%s" % (user, host)) or host
-    if args.startswith('-') or args.startswith('/'):
+    pflag = b'plink' in sshcmd.lower() and b'-P' or b'-p'
+    args = user and (b"%s@%s" % (user, host)) or host
+    if args.startswith(b'-') or args.startswith(b'/'):
         raise error.Abort(
-            _('illegal ssh hostname or username starting with - or /: %s') %
-            args)
+            _(b'illegal ssh hostname or username starting with - or /: %s')
+            % args
+        )
     args = shellquote(args)
     if port:
-        args = '%s %s %s' % (pflag, shellquote(port), args)
+        args = b'%s %s %s' % (pflag, shellquote(port), args)
     return args
 
+
 def setflags(f, l, x):
     pass
 
+
 def copymode(src, dst, mode=None, enforcewritable=False):
     pass
 
+
 def checkexec(path):
     return False
 
+
 def checklink(path):
     return False
 
+
 def setbinary(fd):
     # When run without console, pipes may expose invalid
     # fileno(), usually set to -1.
     fno = getattr(fd, 'fileno', None)
     if fno is not None and fno() >= 0:
-        msvcrt.setmode(fno(), os.O_BINARY)
+        msvcrt.setmode(fno(), os.O_BINARY)  # pytype: disable=module-attr
+
 
 def pconvert(path):
-    return path.replace(pycompat.ossep, '/')
+    return path.replace(pycompat.ossep, b'/')
+
 
 def localpath(path):
-    return path.replace('/', '\\')
+    return path.replace(b'/', b'\\')
+
 
 def normpath(path):
     return pconvert(os.path.normpath(path))
 
+
 def normcase(path):
-    return encoding.upper(path) # NTFS compares via upper()
+    return encoding.upper(path)  # NTFS compares via upper()
+
 
 # see posix.py for definitions
 normcasespec = encoding.normcasespecs.upper
 normcasefallback = encoding.upperfallback
 
+
 def samestat(s1, s2):
     return False
 
+
 def shelltocmdexe(path, env):
     r"""Convert shell variables in the form $var and ${var} inside ``path``
     to %var% form.  Existing Windows style variables are left unchanged.
@@ -318,9 +355,9 @@ def shelltocmdexe(path, env):
     index = 0
     pathlen = len(path)
     while index < pathlen:
-        c = path[index:index + 1]
-        if c == b'\'':   # no expansion within single quotes
-            path = path[index + 1:]
+        c = path[index : index + 1]
+        if c == b'\'':  # no expansion within single quotes
+            path = path[index + 1 :]
             pathlen = len(path)
             try:
                 index = path.index(b'\'')
@@ -329,7 +366,7 @@ def shelltocmdexe(path, env):
                 res += c + path
                 index = pathlen - 1
         elif c == b'%':  # variable
-            path = path[index + 1:]
+            path = path[index + 1 :]
             pathlen = len(path)
             try:
                 index = path.index(b'%')
@@ -340,8 +377,8 @@ def shelltocmdexe(path, env):
                 var = path[:index]
                 res += b'%' + var + b'%'
         elif c == b'$':  # variable
-            if path[index + 1:index + 2] == b'{':
-                path = path[index + 2:]
+            if path[index + 1 : index + 2] == b'{':
+                path = path[index + 2 :]
                 pathlen = len(path)
                 try:
                     index = path.index(b'}')
@@ -358,11 +395,11 @@ def shelltocmdexe(path, env):
             else:
                 var = b''
                 index += 1
-                c = path[index:index + 1]
+                c = path[index : index + 1]
                 while c != b'' and c in varchars:
                     var += c
                     index += 1
-                    c = path[index:index + 1]
+                    c = path[index : index + 1]
                 # Some variables (like HG_OLDNODE) may be defined, but have an
                 # empty value.  Those need to be skipped because when spawning
                 # cmd.exe to run the hook, it doesn't replace %VAR% for an empty
@@ -376,13 +413,19 @@ def shelltocmdexe(path, env):
 
                 if c != b'':
                     index -= 1
-        elif (c == b'~' and index + 1 < pathlen
-              and path[index + 1:index + 2] in (b'\\', b'/')):
-            res += "%USERPROFILE%"
-        elif (c == b'\\' and index + 1 < pathlen
-              and path[index + 1:index + 2] in (b'$', b'~')):
+        elif (
+            c == b'~'
+            and index + 1 < pathlen
+            and path[index + 1 : index + 2] in (b'\\', b'/')
+        ):
+            res += b"%USERPROFILE%"
+        elif (
+            c == b'\\'
+            and index + 1 < pathlen
+            and path[index + 1 : index + 2] in (b'$', b'~')
+        ):
             # Skip '\', but only if it is escaping $ or ~
-            res += path[index + 1:index + 2]
+            res += path[index + 1 : index + 2]
             index += 1
         else:
             res += c
@@ -390,6 +433,7 @@ def shelltocmdexe(path, env):
         index += 1
     return res
 
+
 # A sequence of backslashes is special iff it precedes a double quote:
 # - if there's an even number of backslashes, the double quote is not
 #   quoted (i.e. it ends the quoted region)
@@ -403,6 +447,8 @@ def shelltocmdexe(path, env):
 # quote we've appended to the end)
 _quotere = None
 _needsshellquote = None
+
+
 def shellquote(s):
     r"""
     >>> shellquote(br'C:\Users\xyz')
@@ -432,40 +478,37 @@ def shellquote(s):
         return s
     return b'"%s"' % _quotere.sub(br'\1\1\\\2', s)
 
+
 def _unquote(s):
     if s.startswith(b'"') and s.endswith(b'"'):
         return s[1:-1]
     return s
 
+
 def shellsplit(s):
     """Parse a command string in cmd.exe way (best-effort)"""
     return pycompat.maplist(_unquote, pycompat.shlexsplit(s, posix=False))
 
-def quotecommand(cmd):
-    """Build a command string suitable for os.popen* calls."""
-    if sys.version_info < (2, 7, 1):
-        # Python versions since 2.7.1 do this extra quoting themselves
-        return '"' + cmd + '"'
-    return cmd
 
 # if you change this stub into a real check, please try to implement the
 # username and groupname functions above, too.
 def isowner(st):
     return True
 
+
 def findexe(command):
-    '''Find executable for command searching like cmd.exe does.
+    """Find executable for command searching like cmd.exe does.
     If command is a basename then PATH is searched for command.
     PATH isn't searched if command is an absolute or relative path.
     An extension from PATHEXT is found and added if not present.
-    If command isn't found None is returned.'''
-    pathext = encoding.environ.get('PATHEXT', '.COM;.EXE;.BAT;.CMD')
+    If command isn't found None is returned."""
+    pathext = encoding.environ.get(b'PATHEXT', b'.COM;.EXE;.BAT;.CMD')
     pathexts = [ext for ext in pathext.lower().split(pycompat.ospathsep)]
     if os.path.splitext(command)[1].lower() in pathexts:
-        pathexts = ['']
+        pathexts = [b'']
 
     def findexisting(pathcommand):
-        'Will append extension (if needed) and return existing file'
+        """Will append extension (if needed) and return existing file"""
         for ext in pathexts:
             executable = pathcommand + ext
             if os.path.exists(executable):
@@ -475,57 +518,66 @@ def findexe(command):
     if pycompat.ossep in command:
         return findexisting(command)
 
-    for path in encoding.environ.get('PATH', '').split(pycompat.ospathsep):
+    for path in encoding.environ.get(b'PATH', b'').split(pycompat.ospathsep):
         executable = findexisting(os.path.join(path, command))
         if executable is not None:
             return executable
     return findexisting(os.path.expanduser(os.path.expandvars(command)))
 
+
 _wantedkinds = {stat.S_IFREG, stat.S_IFLNK}
 
+
 def statfiles(files):
-    '''Stat each file in files. Yield each stat, or None if a file
+    """Stat each file in files. Yield each stat, or None if a file
     does not exist or has a type we don't care about.
 
-    Cluster and cache stat per directory to minimize number of OS stat calls.'''
-    dircache = {} # dirname -> filename -> status | None if file does not exist
+    Cluster and cache stat per directory to minimize number of OS stat calls."""
+    dircache = {}  # dirname -> filename -> status | None if file does not exist
     getkind = stat.S_IFMT
     for nf in files:
-        nf  = normcase(nf)
+        nf = normcase(nf)
         dir, base = os.path.split(nf)
         if not dir:
-            dir = '.'
+            dir = b'.'
         cache = dircache.get(dir, None)
         if cache is None:
             try:
-                dmap = dict([(normcase(n), s)
-                             for n, k, s in listdir(dir, True)
-                             if getkind(s.st_mode) in _wantedkinds])
+                dmap = {
+                    normcase(n): s
+                    for n, k, s in listdir(dir, True)
+                    if getkind(s.st_mode) in _wantedkinds
+                }
             except OSError as err:
                 # Python >= 2.5 returns ENOENT and adds winerror field
                 # EINVAL is raised if dir is not a directory.
-                if err.errno not in (errno.ENOENT, errno.EINVAL,
-                                     errno.ENOTDIR):
+                if err.errno not in (errno.ENOENT, errno.EINVAL, errno.ENOTDIR):
                     raise
                 dmap = {}
             cache = dircache.setdefault(dir, dmap)
         yield cache.get(base, None)
 
+
 def username(uid=None):
     """Return the name of the user with the given uid.
 
     If uid is None, return the name of the current user."""
+    if not uid:
+        return pycompat.fsencode(getpass.getuser())
     return None
 
+
 def groupname(gid=None):
     """Return the name of the group with the given gid.
 
     If gid is None, return the name of the current group."""
     return None
 
+
 def readlink(pathname):
     return pycompat.fsencode(os.readlink(pycompat.fsdecode(pathname)))
 
+
 def removedirs(name):
     """special version of os.removedirs that does not remove symlinked
     directories or junction points if they actually contain files"""
@@ -544,6 +596,7 @@ def removedirs(name):
             break
         head, tail = os.path.split(head)
 
+
 def rename(src, dst):
     '''atomically rename file src to dst, replacing dst if it exists'''
     try:
@@ -554,16 +607,20 @@ def rename(src, dst):
         unlink(dst)
         os.rename(src, dst)
 
+
 def gethgcmd():
     return [encoding.strtolocal(arg) for arg in [sys.executable] + sys.argv[:1]]
 
+
 def groupmembers(name):
     # Don't support groups on Windows for now
     raise KeyError
 
+
 def isexec(f):
     return False
 
+
 class cachestat(object):
     def __init__(self, path):
         pass
@@ -571,15 +628,16 @@ class cachestat(object):
     def cacheable(self):
         return False
 
+
 def lookupreg(key, valname=None, scope=None):
-    ''' Look up a key/value name in the Windows registry.
+    """Look up a key/value name in the Windows registry.
 
     valname: value name. If unspecified, the default value for the key
     is used.
     scope: optionally specify scope for registry lookup, this can be
     a sequence of scopes to look up in order. Default (CURRENT_USER,
     LOCAL_MACHINE).
-    '''
+    """
     if scope is None:
         scope = (winreg.HKEY_CURRENT_USER, winreg.HKEY_LOCAL_MACHINE)
     elif not isinstance(scope, (list, tuple)):
@@ -594,20 +652,25 @@ def lookupreg(key, valname=None, scope=None):
         except EnvironmentError:
             pass
 
+
 expandglobs = True
 
+
 def statislink(st):
     '''check whether a stat result is a symlink'''
     return False
 
+
 def statisexec(st):
     '''check whether a stat result is an executable file'''
     return False
 
+
 def poll(fds):
     # see posix.py for description
     raise NotImplementedError()
 
+
 def readpipe(pipe):
     """Read all available data from a pipe."""
     chunks = []
@@ -621,7 +684,8 @@ def readpipe(pipe):
             break
         chunks.append(s)
 
-    return ''.join(chunks)
+    return b''.join(chunks)
+
 
 def bindunixsocket(sock, path):
-    raise NotImplementedError(r'unsupported platform')
+    raise NotImplementedError('unsupported platform')
